#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import implementer, provider
from zope.schema.fieldproperty import FieldProperty

from pyams_content.component.illustration.interfaces import IIllustrationTarget, ILinkIllustrationTarget
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget
from pyams_content.component.theme.interfaces import ICollectionsTarget, ITagsTarget, IThemesTarget
from pyams_content.features.preview.interfaces import IPreviewTarget
from pyams_content.features.review.interfaces import IReviewTarget
from pyams_content.shared.common import SharedContent, register_content_type
from pyams_content.shared.common.interfaces import IWfSharedContentFactory
from pyams_content.shared.common.types import WfTypedSharedContent
from pyams_content.shared.resource.interfaces import IWfResource, RESOURCE_CONTENT_TYPE, RESOURCE_CONTENT_NAME, IResourceInfo, \
    RESOURCE_INFO_ANNOTATIONS_KEY, IWfResourceFactory, IResource
from pyams_utils.adapter import adapter_config, get_annotation_adapter


@implementer(IWfResource, IIllustrationTarget, ILinkIllustrationTarget, IParagraphContainerTarget,
             ITagsTarget, ICollectionsTarget, IThemesTarget, IPreviewTarget, IReviewTarget)
class WfResource(WfTypedSharedContent):
    """Resource class"""

    content_type = RESOURCE_CONTENT_TYPE
    content_name = RESOURCE_CONTENT_NAME


register_content_type(WfResource)


@implementer(IResourceInfo)
class ResourceInfo(Persistent, Contained):
    """Resource persistent informations"""

    original_country = FieldProperty(IResourceInfo['original_country'])
    author = FieldProperty(IResourceInfo['author'])
    drawer = FieldProperty(IResourceInfo['drawer'])
    colourist = FieldProperty(IResourceInfo['colourist'])
    lettering = FieldProperty(IResourceInfo['lettering'])
    producer = FieldProperty(IResourceInfo['producer'])
    director = FieldProperty(IResourceInfo['director'])
    actors = FieldProperty(IResourceInfo['actors'])
    editor = FieldProperty(IResourceInfo['editor'])
    collection = FieldProperty(IResourceInfo['collection'])
    series = FieldProperty(IResourceInfo['series'])
    volume = FieldProperty(IResourceInfo['volume'])
    format = FieldProperty(IResourceInfo['format'])
    release_year = FieldProperty(IResourceInfo['release_year'])
    awards = FieldProperty(IResourceInfo['awards'])
    nb_pages = FieldProperty(IResourceInfo['nb_pages'])
    editor_reference = FieldProperty(IResourceInfo['editor_reference'])
    isbn_number = FieldProperty(IResourceInfo['isbn_number'])
    source_url = FieldProperty(IResourceInfo['source_url'])
    price = FieldProperty(IResourceInfo['price'])
    summary = FieldProperty(IResourceInfo['summary'])
    synopsis = FieldProperty(IResourceInfo['synopsis'])
    publisher_words = FieldProperty(IResourceInfo['publisher_words'])


@adapter_config(context=IWfResource, provides=IResourceInfo)
def resource_info_factory(context):
    """Resource info factory"""
    return get_annotation_adapter(context, RESOURCE_INFO_ANNOTATIONS_KEY, ResourceInfo, name='++info++')


@provider(IWfResourceFactory)
@implementer(IResource)
class Resource(SharedContent):
    """Workflow managed resource"""


@adapter_config(context=IWfResourceFactory, provides=IWfSharedContentFactory)
def resource_content_factory(context):
    return WfResource
